/*
  This file is part of the NetFax system.

  (c) Copyright 1989 by David M. Siegel. 
      All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation.

    This program is distributed in the hope that it will be useful, 
    but WITHOUT ANY WARRANTY; without even the implied warranty of 
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdio.h>
#include <signal.h>
#ifdef VMS
#include <file.h>
#else	/* VMS */
#include <fcntl.h>
#include <pwd.h>
#include <sys/param.h>
#endif	/* VMS */

#include "../../lib/libutil/list.h"
#include "../../lib/libfax/libfax.h"

/*
 * Some flags that we share with the signal handlers.
 */
static int pages = -1;
static int coversheet = FALSE;

/*
 * Base filenames for the PPM files that Ghostscript generates.
 */
static char out_base[MAXPATHLEN];
static char coversheet_base[MAXPATHLEN];

/*
 * This is the Postscript file that is being spooled.
 */
static int delete_ps_file = FALSE;
static char *ps_file;

/*
 * Delete generated G3 files.
 */
static void cleanup_g3_files(base, pages)
     char *base;
     int pages;
{
    int page;

    for (page = 1; page <= pages; page++) {
	char file[MAXPATHLEN];
#ifdef VMS
	sprintf(file, "%s.g3_%d", base, page);
#else	/* VMS */
	sprintf(file, "%s.g3.%d", base, page);
#endif	/* VMS */
	unlink(file);
    }
}

/*
 * Cleanup temporary files.
 */
static void cleanup()
{
    if (delete_ps_file)
      unlink(ps_file);
    if (pages > 0 && strlen(out_base) > 0)
      cleanup_g3_files(out_base, pages);
    if (coversheet && strlen(coversheet_base) > 0)
      cleanup_g3_files(coversheet_base, 1);
}

/*
 * Called when we get a variety of signals.
 */
static void signal_handler()
{
    fprintf(stderr, "transmission aborted\n");
    cleanup();
    exit(1);
}

/*
 * Install the signals that we want to trap for cleanup.
 */
static void install_signal_handlers()
{
    signal(SIGHUP, signal_handler);
    signal(SIGINT, signal_handler);
    signal(SIGQUIT, signal_handler);
}

main(argc,argv)
     int argc;
     char *argv[];
{
    int c;
    int errflg = 0;
    extern char *optarg;
    extern int optind;

    int pid = getpid();
    int now = time(0);

    int got_recip = 0;
    int rc;

    char fax_cmd[2048];
    char recips[2048];
    char *recipient = "";
    char *sender = "";
    char *sender_fax = "";
    int mail_notification = FALSE;
    char *host = NULL;
    char *user_name = NULL;
    char *SubjectA, *SubjectB, *SubjectC, *SubjectD;
    int	hour = 0;	/* After which hour to send? */

    strcpy(recips, "");
    strcpy(out_base, "");
    strcpy(coversheet_base, "");

    SubjectA = SubjectB = SubjectC = SubjectD = NULL;

    while ((c = getopt(argc, argv, "cp:r:s:S:mu:h:A:B:C:D:a:")) != -1)
      switch (c) {
	case 'c':
	  coversheet = TRUE;
	  break;
	case 'p':
	  got_recip++;
	  strcat(recips, " ");
	  strcat(recips, optarg);
	  break;
	case 'r':
	  recipient = optarg;
	  break;
	case 's':
	  sender = optarg;
	  break;
	case 'S':
	  sender_fax = optarg;
	  break;
	case 'm':
	  mail_notification = TRUE;
	  break;
	case 'u':
	  user_name = optarg;
	  break;
	case 'h':
	  host = optarg;
	  break;
	case 'A':
		SubjectA = optarg; break;
	case 'B':
		SubjectB = optarg; break;
	case 'C':
		SubjectC = optarg; break;
	case 'D':
		SubjectD = optarg; break;
	case 'a':
		hour = atoi(optarg); break;
	case '?':
	  errflg++;;
	  break;
      }

    if (errflg || got_recip == 0) {
	fprintf(stderr, "usage: %s -p phone [filename]\n", argv[0]);
	fprintf(stderr, "\tone or more -p argument may be given\n");
	fprintf(stderr, "\tif no filename is given, input is from stdin\n");
	fprintf(stderr, "optional args:\n");
	fprintf(stderr, "\t-m will send email notification on delivery\n");
	fprintf(stderr, "\t-h <host> uses the spooler on the given host\n");
	fprintf(stderr, "\t-u <user> spools job with the given user name\n");
	fprintf(stderr, "\t-c generates a coverpage\n");
	fprintf(stderr, "\t-s <name> specifies the coverpage sender\n");
	fprintf(stderr, "\t-S <phone> specifies the coverpage return fax\n");
	fprintf(stderr, "\t-r <name> specifies the coverpage recipient\n");
	fprintf(stderr, "\t-A 'text' - First line of subject\n");
	fprintf(stderr, "\t->B to D 'text' - Second to Fourth lines of subject\n");
	fprintf(stderr, "\t-a <hour> - Send after hour. Hour is in the range 0 to 23\n");
	exit(1);
    }

    /*
     * Catch some signals so we can do a proper cleanup.
     */
    install_signal_handlers();

    /*
     * Do we get input from stdin?
     */
    if (optind == argc) {
	char *mktemp();
	char buf[BUFSIZ];
	int bytes;
	int fd;

#ifdef VMS	
	ps_file = mktemp("/fax_root/TMP/faxdXXXXXX");
#else	/* VMS */
	ps_file = mktemp("/tmp/faxdXXXXXX");
#endif	/* VMS */
	delete_ps_file = TRUE;

	if ((fd = open(ps_file, O_WRONLY|O_CREAT, 0666)) < 0) {
	    perror("temp file");
	    exit(2);
	}

	while ((bytes = read(0, buf, sizeof(buf))) > 0)
	  write(fd, buf, bytes);
	
	close(fd);
    } else {
/* Copy, add our header and replace each showpage command with ShowPage one */
	char *mktemp();
	char buf[BUFSIZ];
	int bytes;
	FILE	*Ifd, *Ofd;

#ifdef VMS	
	ps_file = mktemp("/fax_root/TMP/faxdXXXXXX");
#else	/* VMS */
	ps_file = mktemp("/tmp/faxdXXXXXX");
#endif	/* VMS */
	delete_ps_file = TRUE;

	if((Ifd = fopen(argv[optind], "r")) == NULL) {
		perror(argv[optind]); exit(2);
	}
	if((Ofd = fopen(ps_file, "w")) == NULL) {
		perror(ps_file); exit(2);
	}

	fprintf(Ofd, "%%!PS-Adobe-2.0\n");
	fprintf(Ofd, "/Ypages 0 def\n");
	fprintf(Ofd, "/ShowPage { Ypages 1 add dup /Ypages exch store (     ) cvs\n");
	fprintf(Ofd, "gsave 36 780 moveto\n");
	fprintf(Ofd, "/Courier-BoldOblique findfont 10 scalefont setfont\n");
	fprintf(Ofd, "(The Hebrew University    Modified NetFax+Dynamode-M14;   Page )\
show show grestore showpage } def\n\n");

	while(fgets(buf, sizeof(buf) - 1, Ifd) != NULL) {
		char	*p, *q;
		q = buf;

		/* Look for showpage */
		while(q != NULL) {
			if((p = strchr(q, 's')) != NULL) {
				if(strncmp(p, "showpage", 8) == 0) {
					p[0] = 'S'; p[4] = 'P';
				}
			}
			if(p == NULL)
				q = p;
			else	q = &p[1];
		}
		fprintf(Ofd, "%s", buf);
	}

	fclose(Ifd); fclose(Ofd);
    }

    /*
     * Make the G3 files.
     */
#ifdef VMS
    sprintf(out_base, "/fax_root/TMP/fax%d%d", pid, now);
#else	/* VMS */
    sprintf(out_base, "/tmp/fax%d%d", pid, now);
#endif /* VMS */
     printf("Converting PostScript file to Fax format. Please wait...  ");
    if ((pages = cvt_postscript_to_g3(ps_file, out_base)) <= 0) {
	fprintf(stderr, "can't convert postscript file\n");
	exit(3);
    }
    printf("Done, %d pages\n", pages);

    /*
     * Make the coversheet.
     */
    if (coversheet) {
#ifdef VMS
	sprintf(coversheet_base, "/fax_root/TMP/faxc%d%d", pid, now);
#else	/* VMS */
	sprintf(coversheet_base, "/tmp/faxc%d%d", pid, now);
#endif	/* VMS */
	printf("Creating cover page. Please wait...  ");
	if (cvt_coversheet_to_g3(coversheet_base, recipient, sender, 
				 sender_fax, pages,
				 SubjectA, SubjectB, SubjectC, SubjectD) < 0) {
	    fprintf(stderr, "can't convert coversheet\n");
	    cleanup_g3_files(out_base, pages);
	    exit(3);
	}
	printf("Done\n");
    }

    /*
     * Build the fax command.
     */
    sprintf(fax_cmd, "%s", FAX_ENQ_PROG);
    if (user_name != NULL)
      sprintf(&fax_cmd[strlen(fax_cmd)], " -u %s", user_name);
    if (host != NULL)
      sprintf(&fax_cmd[strlen(fax_cmd)], " -h %s", host);
    if (mail_notification)
      strcat(fax_cmd, " -m");
    if (hour)
	sprintf(&fax_cmd[strlen(fax_cmd)], " -a %d", hour);
    strcat(fax_cmd, recips);
    strcat(fax_cmd, " ---");
    if (coversheet)
#ifdef VMS
      sprintf(&fax_cmd[strlen(fax_cmd)], " %s.g3_1", coversheet_base);
#else	/* VMS */
      sprintf(&fax_cmd[strlen(fax_cmd)], " %s.g3.1", coversheet_base);
#endif	/* VMS */
    sprintf(&fax_cmd[strlen(fax_cmd)], " %s.g3:%d", out_base, pages);

    /*
     * Run the fax command.
     */
    printf("Sending to fax spooler; please wait...\n");
    rc = system(fax_cmd);

    /*
     * All done!  Cleanup and exit.
     */
    cleanup();

    exit(rc);
}


#ifdef VMS
log() {}
#endif
