/*
  This file is part of the NetFax system.

  (c) Copyright 1989 by David M. Siegel. 
      All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation.

    This program is distributed in the hope that it will be useful, 
    but WITHOUT ANY WARRANTY; without even the implied warranty of 
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdio.h>
#ifdef VMS
#include <file.h>
#else	/* VMS */
#include <fcntl.h>
#include <unistd.h>
#endif	/* VMS */

#include "../../lib/libfax/libfax.h"

static int verbose = FALSE;
FILE	*info_fp;

#define MAX_TRIES  3

#define EXIT_ERROR		2
#define EXIT_NO_FILE		4
#define EXIT_OPEN_FAILED	6
#define EXIT_CALL_FAILED	8
#define EXIT_SEND_FAILED	10
#define EXIT_FINISH_FAILED	12


static void print_fdis_params(f, fp)
     FaxModem *f;
     FILE *fp;
{
    if (FAX_ISSET(f, FAX_F_FDCS)) {
	fprintf(fp, " vr=%s br=%s wd=%s ln=%s,ecm=%s\n",
		t30_vr_string(&f->fdis_params),
		t30_br_string(&f->fdis_params),
		t30_wd_string(&f->fdis_params),
		t30_ln_string(&f->fdis_params),
		t30_ec_string(&f->fdis_params));
      }
}


static int do_send(fmp, fd, page, last_page)
     FaxModem *fmp;
     int fd;
     int page;
     int last_page;
{
    long start = time(0);

    if (faxmodem_send_page(fmp, fd, last_page, MAX_TRIES) < 0)
      return (-1);

    if (verbose)
      printf("page %d sent in %d seconds\n", page, time(0) - start);

    return (0);
}

main(argc, argv)
     int argc;
     char *argv[];
{
    int c;
    extern char *optarg;
    extern int optind;
    int errflg = 0, pages;

    char *fax_device = FAX_DEVICE;
    char *phone;
    FaxModem fm;
    long start;

    log_set_level(LOG_WARNING);

    while ((c = getopt(argc, argv, "l:vf:")) != -1)
      switch (c) {
	case 'l':
	  log_set_level(atoi(optarg));
	  break;
	case 'v':
	  verbose = TRUE;
	  break;
	case 'f':
	  fax_device = optarg;
	  break;
	case '?':
	  errflg++;
	  break;
      }

    if (errflg || optind >= argc) {
	fprintf(stderr, "usage: %s phone [files..]\n", argv[0]);
	exit(EXIT_ERROR);
    }

/* Open the info file */
    {
	char	filename[128];
	start = time(0);
	sprintf(filename, "%s/FAX.LOG", OUTGOING_QUEUE);
	info_fp = fopen(filename, "a+");
	fprintf(info_fp, "Transmission of fax started at %s", ctime(&start));
    }

    phone = argv[optind++];
    fm.hangup_code = 256;	/* Init it to some known value */

    if (faxmodem_open(&fm, fax_device) < 0) {
	fprintf(stderr, "open of fax failed\n");
	exit(EXIT_OPEN_FAILED);
    }

    if (faxmodem_initiate_call(&fm, phone) < 0 || !FAX_CONNECTED(&fm)) {
	fprintf(stderr, "call to %s failed, dial code=%d, hangup=%d\n",
		phone, fm.dialer_code, fm.hangup_code);
	fprintf(info_fp, "call to %s failed, dial code=%d, hangup=%d\n",
		phone, fm.dialer_code, fm.hangup_code);
	exit(EXIT_CALL_FAILED);
    }

    start = time(0);

    if (verbose) {
	printf("connected to %s\n", phone);
	faxmodem_print_id_strings(&fm, stdout);
    }
    fprintf(info_fp, "connected to %s\n", phone);
    faxmodem_print_id_strings(&fm, info_fp);
    print_fdis_params(&fm, info_fp);

    if (optind < argc) {
	int page = 1;
	for (; optind < argc; optind++) {
	    int fd;

/* HUJI - If the filename contains : then we loop on that name and number */
	    if(strchr(argv[optind], ':') != NULL) {
		char	*p, FilenameBase[128], Filename[128];
		int	i;

		strcpy(FilenameBase, argv[optind]);	/* Do not ruin it */
		*(p = strchr(FilenameBase, ':')) = '\0'; p++;
		pages = atoi(p);
		for(i = 0; i < pages; i++) {
			sprintf(Filename, "%s_%d.g3", FilenameBase, i);
			if ((fd = open(Filename, O_RDONLY)) < 0) {
				fprintf(stderr, "can't access file: %s\n", Filename);
				exit(EXIT_NO_FILE);
			}
			if (do_send(&fm, fd, i, i+1 == pages) < 0) {
				fprintf(stderr, "sending failed on: %s\n",
					Filename);
				fprintf(info_fp, "sending failed on page %d, hangup=%d\n",
					i, fm.hangup_code);
				exit(EXIT_SEND_FAILED);
			}
		}
	    } else {
		if ((fd = open(argv[optind], O_RDONLY)) < 0) {
			fprintf(stderr, "can't access file: %s\n", argv[optind]);
			exit(EXIT_NO_FILE);
		}
		if (do_send(&fm, fd, page++, optind+1 == argc) < 0) {
			fprintf(stderr, "sending failed on: %s, hangup=%d\n",
				argv[optind], fm.hangup_code);
			fprintf(info_fp, "sending failed on: %s, hangup=%d\n",
				argv[optind], fm.hangup_code);
			exit(EXIT_SEND_FAILED);
		}
            }
	}
    } else
      if (do_send(&fm, 0, 1, TRUE) < 0) {
	  fprintf(stderr, "sending failed stdin\n");
	  exit(EXIT_SEND_FAILED);
      }
      

    if (faxmodem_hangup(&fm) < 0) {
	fprintf(stderr, "hangup failed\n");
	fprintf(info_fp, "Hangup failed, code=%d\n", fm.hangup_code);
	exit(EXIT_FINISH_FAILED);
    }

    if (verbose)
      printf("total connect time was %d seconds\n\n", time(0) - start);
    fprintf(info_fp, "%d pages sent, connect time: %d seconds, Hangup=%d\n",
		pages, time(0) - start, fm.hangup_code);

    if (faxmodem_close(&fm) < 0) {
	fprintf(stderr, "close failed\n");
	exit(EXIT_FINISH_FAILED);
    }

    fclose(info_fp);
    exit(1);
}

#ifdef VMS
openlog() {}
syslog() {}
#endif
